local state = require("loaded_state")
local snapshot = require("structs.snapshot")
local utils = require("utils")
local mapcoder = require("mapcoder")
local mapStruct = require("structs.map")
local mapItemUtils = require("map_item_utils")
local celesteRender = require("celeste_render")

local script = {
    name = "copyStylegrounds",
    displayName = "Copy Stylegrounds From Map",
    parameters = {
        from = "",
        group = "",
        tagsToAdd = "",
        onlyPrefix = "",

        offsetX = 0,
        offsetY = 0,
    },
    fieldInformation = {
        from = {
            fieldType = "loennScripts.directFilepath",
            extension = "bin",
        }
    },
    tooltip = "Copies a whole map into this one",
    tooltips = {
        from = "The file to copy",
        group = "The name of the group to put all these stylegrounds in (leave this blank, and they won't go in a group)",
        tagsToAdd = "A comma-separated list of tags to add to the stylegrounds",
        onlyPrefix = "The prefix to add to all entries in the \"Only\" list",
        offsetX = "The destination map X minus the source map X",
        offsetY = "The destination map Y minus the source map Y",
    },
}

function process(styleground, args)
    if args.tagsToAdd ~= "" then
        styleground.tag = style.tag or ""
        if styleground.tag ~= "" then
            styleground.tag = style.tag .. ","
        end
        styleground.tag = styleground.tag .. args.tagsToAdd
    end

    if styleground._type == "parallax" then
        styleground.x = styleground.x + args.offsetX * 8 * styleground.scrollX
        styleground.y = styleground.y + args.offsetY * 8 * styleground.scrollY
    end

    if styleground.only ~= nil then
        local newOnly = ""
        for part in styleground.only:gmatch("([^,]+)") do
            if newOnly ~= "" then newOnly = newOnly .. "," end
            newOnly = args.onlyPrefix .. part
        end
        styleground.only = newOnly
    end
end

function script.prerun(args, mode, ctx)
    local targetMap = mapStruct.decode(mapcoder.decodeFile(args.from))
    local stylesFg = utils.deepcopy(targetMap.stylesFg)
    local stylesBg = utils.deepcopy(targetMap.stylesBg)

    for _, styleground in ipairs(stylesFg) do
        process(style, args)
    end
    for _, styleground in ipairs(stylesBg) do
        process(style, args)
    end

    local function forward(data)
        for _, style in ipairs(stylesFg) do
            state.map.stylesFg:insert(style)
        end

        for _, style in ipairs(stylesBg) do
            state.map.stylesBg:insert(style)
        end
    end

    local function backward(data)
        for _, remove in ipairs(stylesFg) do
            for i, style in ipairs(state.map.stylesFg) do
                if style == remove then
                    state.map.stylesFg:remove(i)
                end
            end
        end

        for _, remove in ipairs(stylesBg) do
            for i, style in ipairs(state.map.stylesBg) do
                if style == remove then
                    state.map.stylesBg:remove(i)
                end
            end
        end
    end

    forward()

    return snapshot.create(script.name, {}, backward, forward)
end

return script
